@@ -9,17 +9,29 @@
  * it was previously.
  *
  * Wed Sep 14 22:43:00 1994: Mitchum DSouza
- * (mitch@mrc-applied-psychology.cambridge.ac.uk) added support for mouting
+ * (mitch@mrc-applied-psychology.cambridge.ac.uk) added support for mounting
  * the "loop" device.
  *
  * Wed Sep 14 22:55:10 1994: Sander van Malssen (svm@kozmix.hacktic.nl)
  * added support for remounting readonly file systems readonly.
  *
+ * Wed Feb 8 09:23:18 1995: Mike Grupenhoff <kashmir@umiacs.UMD.EDU> added
+ * a probe of the superblock for the type before /proc/filesystems is
+ * checked.
+ *
+ * Wed Feb  8 12:27:00 1995: Andries.Brouwer@cwi.nl fixed up error messages.
+ *
  */
 
 #include "sundries.h"
 
 #include <linux/fs.h>
+#include <linux/minix_fs.h>
+#include <linux/ext_fs.h>
+#include <linux/ext2_fs.h>
+#include <linux/xia_fs.h>
+#include <sys/stat.h>
+#include <unistd.h>
 
 int del_loop (const char *);
 
@@ -76,8 +88,8 @@ const struct opt_map opt_map[] =
   { "nosuid",	0, MS_NOSUID	},	/* don't honor suid executables */
   { "dev",	1, MS_NODEV	},	/* interpret device files  */
   { "nodev",	0, MS_NODEV	},	/* don't interpret devices */
-  { "sync",	0, MS_SYNC	},	/* synchronous I/O */
-  { "async",	1, MS_SYNC	},	/* asynchronous I/O */
+  { "sync",	0, MS_SYNCHRONOUS},	/* synchronous I/O */
+  { "async",	1, MS_SYNCHRONOUS},	/* asynchronous I/O */
   { "remount",  0, MS_REMOUNT   },      /* Alter flags of mounted FS */
   { "auto",	1, MS_NOAUTO	},	/* Can be mounted using -a */
   { "noauto",	0, MS_NOAUTO	},	/* Can  only be mounted explicitly */
@@ -202,6 +214,65 @@ fix_opts_string (int flags, char *extra_opts)
   return new_opts;
 }
 
+
+/*
+    char *fstype(const char *device);
+
+    probes the device and attempts to determine the type of filesystem
+    contained within.
+
+    Original routine by <jmorriso@bogomips.ww.ubc.ca>; made into a function
+    for mount(8) by Mike Grupenhoff <kashmir@umiacs.umd.edu>.
+
+    Currently supports: minix, ext, ext2, xia
+*/
+
+static char *
+fstype(const char *device)
+{
+    int fd;
+
+    /* MINIX */
+    struct minix_super_block ms;
+    /* extended fs */
+    struct ext_super_block es;
+    /* 2nd extended fs */
+    struct ext2_super_block e2s;
+    /* xia fs */
+    struct xiafs_super_block xfs;
+
+    fd = open(device, O_RDONLY);
+    if (fd < 0) {
+	perror(device);
+	exit(1);
+    }
+    lseek(fd, BLOCK_SIZE, SEEK_SET);
+    read(fd, (char *) &ms, sizeof(ms));
+    if (ms.s_magic == MINIX_SUPER_MAGIC || ms.s_magic == MINIX_SUPER_MAGIC2)
+	return("minix");
+
+    lseek(fd, BLOCK_SIZE, SEEK_SET);
+    read(fd, (char *) &es, sizeof(es));
+    if (es.s_magic == EXT_SUPER_MAGIC)
+	return("ext");
+
+    lseek(fd, BLOCK_SIZE, SEEK_SET);
+    read(fd, (char *) &e2s, sizeof(e2s));
+    if (e2s.s_magic == EXT2_SUPER_MAGIC || e2s.s_magic == EXT2_PRE_02B_MAGIC)
+	return("ext2");
+
+    lseek(fd, 0, SEEK_SET);
+    read(fd, (char *) &xfs, sizeof(xfs));
+    if (xfs.s_magic == _XIAFS_SUPER_MAGIC)
+	return("xiafs");
+
+    close(fd);
+
+    return(0);
+
+}
+
+
 /* Mount a single file system.  Return status,
    so don't exit on non-fatal errors.  */
 
@@ -285,6 +356,9 @@ mount_one (char *spec, char *node, char *type, char *opts, int freq, int pass)
     die (1, "mount: this version doesn't support the type `nfs'");
 #endif
 
+  if (!type)
+  	type = fstype(spec);
+
   block_signals (SIG_BLOCK);
 
   if (fake
@@ -339,15 +413,24 @@ mount_one (char *spec, char *node, char *type, char *opts, int freq, int pass)
 	error ("mount: must be superuser to use mount");
       break;
     case EBUSY:
-      error ("mount: wrong fs type, %s already mounted, %s busy, "
-	"or other error", spec, node);
+      error ("mount: %s already mounted or %s busy", spec, node);
       break;
     case ENOENT:
-      error ("mount: mount point %s does not exist", node); break;
+      { struct stat statbuf;
+	if (stat (node, &statbuf))
+	      error ("mount: mount point %s does not exist", node);
+	else if (stat (spec, &statbuf))
+	      error ("mount: special device %s does not exist", spec);
+	else {
+           errno = mnt_err;
+           perror("mount");
+	}
+	break;
+     }
     case ENOTDIR:
       error ("mount: mount point %s is not a directory", node); break;
     case EINVAL:
-      error ("mount: %s not a mount point", spec); break;
+      error ("mount: wrong fs type or bad superblock on %s", spec); break;
     case EMFILE:
       error ("mount table full"); break;
     case EIO:
